import 'package:flutter/material.dart';

enum AnimationType {
  normal,
  fadeIn,
  slideTransition,
  scaleTransition,
  rotationTransition
}

class RouteTansitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animationType;
  final Duration duration;
  final bool replacement;

  RouteTansitions({
    required this.context,
    required this.child,
    this.animationType = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
    this.replacement = false,
  }) {
    switch (animationType) {
      case AnimationType.normal:
        _normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
      case AnimationType.slideTransition:
        _slideTransition();
        break;
      case AnimationType.scaleTransition:
        _scaleTransition();
        break;
      case AnimationType.rotationTransition:
        _rotationTransition();
        break;
    }
    // Navigator.push(context, MaterialPageRoute(builder: (context) => child));
  }
  void pushPage(Route route) => Navigator.push(context, route);

  void pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  void _normalTransition() {
    // Navigator.push(context, MaterialPageRoute(builder: (context) => child));
    final route = MaterialPageRoute(builder: (context) => child);
    (replacement)
        ? Navigator.pushReplacement(context, route)
        : Navigator.push(context, route);
  }

  void _fadeInTransition() {
    // final Duration duration =
    //     this.duration ?? const Duration(milliseconds: 300);
    final route = PageRouteBuilder(
      pageBuilder: (BuildContext context, Animation<double> animation,
              Animation<double> secondaryAnimation) =>
          child,
      transitionDuration: duration,
      transitionsBuilder: (
        BuildContext context,
        Animation<double> animation,
        Animation<double> secondaryAnimation,
        Widget child,
      ) {
        return FadeTransition(
          opacity: Tween(
            // Crear animacion
            begin: 0.0,
            end: 1.0,
          ).animate(
            CurvedAnimation(
              parent: animation,
              curve: Curves.easeOut,
            ),
          ),
          child: child,
        );
      },
    );
    // Navigator.push(context, route);
    (replacement)
        ? Navigator.pushReplacement(context, route)
        : Navigator.push(context, route);
  }

  void _slideTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (BuildContext context, Animation<double> animation,
              Animation<double> secondaryAnimation) =>
          child,
      transitionDuration: duration,
      transitionsBuilder: (
        BuildContext context,
        Animation<double> animation,
        Animation<double> secondaryAnimation,
        Widget child,
      ) {
        return SlideTransition(
          position:
              Tween<Offset>(begin: const Offset(0.0, -1.0), end: Offset.zero)
                  .animate(
            CurvedAnimation(
              parent: animation,
              curve: Curves.easeOut,
            ),
          ),
          child: child,
        );
      },
    );

    (replacement)
        ? Navigator.pushReplacement(context, route)
        : Navigator.push(context, route);
  }

  void _scaleTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (BuildContext context, Animation<double> animation,
              Animation<double> secondaryAnimation) =>
          child,
      transitionDuration: duration,
      transitionsBuilder: (
        BuildContext context,
        Animation<double> animation,
        Animation<double> secondaryAnimation,
        Widget child,
      ) {
        return ScaleTransition(
            child: child,
            scale: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(
                parent: animation,
                curve: Curves.easeOut,
              ),
            ));
      },
    );

    (replacement)
        ? Navigator.pushReplacement(context, route)
        : Navigator.push(context, route);
  }

  void _rotationTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (BuildContext context, Animation<double> animation,
              Animation<double> secondaryAnimation) =>
          child,
      transitionDuration: duration,
      transitionsBuilder: (
        BuildContext context,
        Animation<double> animation,
        Animation<double> secondaryAnimation,
        Widget child,
      ) {
        return RotationTransition(
          child: child,
          turns: Tween<double>(begin: 0.0, end: 1.0).animate(
            CurvedAnimation(
              parent: animation,
              curve: Curves.easeOut,
            ),
          ),
        );
      },
    );
    (replacement)
        ? Navigator.pushReplacement(context, route)
        : Navigator.push(context, route);
  }
}
